//**************************************************************************//
//* FRIDA: fast reliable interactive data analysis                         *//
//* coord.cpp: coordinate names and units                                  *//
//* (C) Joachim Wuttke 1990-, v2(C++) 2001-                                *//
//* http://frida.sourceforge.net                                           *//
//**************************************************************************//

#include <math.h>
#include <stdlib.h>
using namespace std;

#include "mystd.h"
#include "coord.h"
#include "range.h"
#include "gar.h"

CCoord::CCoord(void)
{
}
	
CCoord::CCoord( const char *nin, const char *uin)
{
    name = string(nin);
    unit = string(uin);
}	

CCoord::CCoord( const string *nin, const string *uin)
{
    name = *nin;
    unit = *uin;
}	

CCoord::CCoord( const string nin, const string uin)
{
    name = nin;
    unit = uin;
}	

char CoordDecode(const char *resp, CCoord *rval, const CAskPars *AP)
{
    if (resp[0]=='\0') {
        return 'd';
    } else if (!strcmp(resp, "?")) {
        return 'h';
    } else if (!strcmp(resp,"\\EOL")) {
        *rval = CCoord();
        return 'v';
    }

    uint i, j, np, n;
    n = strlen(resp);
    if (resp[n-1]!=')') {
        return 'h';
    }
    for(j=n-2, np=1; j!=(uint)-1; j--) {
        if (resp[j]==')') 
            np++;
        else if (resp[j]=='(') {
            np--;
            if (np==0) break;
        }
    }
    if (np!=0) return 'h';

    for (i=j; i>0 && strchr(" \t",resp[i-1]); i--);
    if (i==0) return 'h';
	
    *rval = CCoord(string(resp, i), string(resp+j+1, n-j-2));
    return 'v';
}

const char* CCoord::helptext = 
"a coordinate: <name> (<unit>)";

CCoord CCoord::Ask(const char *quest, int defflag, const CCoord& defval)
{
    *this =  myask(string(quest), defflag, defval,
                   helptext, CoordDecode, 0, 0);
    return *this;
}

string CCoord::str(void) const
{
    return name + ( unit!="" ? (" (" + unit + ")") : "");
}

string CCoord::ps_str(void) const // just two blanks instead of one
{
    return name + ( unit!="" ? ("  (" + unit + ")") : "");
}

string CCoord::fstr(void) const
{
    return name + "\t" + unit;
}

string CCoord::load(string in)
{
    string::size_type t1, t2;
    if(!in.size()) {
        name = "";
        unit = "";
        return "";
    }
    if ((t1 = in.find('\t'))==string::npos) {
        name = in;
        unit = "";
        return "";
    }
    name = in.substr(0,t1);
    if ((t2 = in.find('\t', t1+1))==string::npos) {
        unit = in.substr(t1+1);
        return "";
    }
    unit = in.substr(t1+1,t2-t1);
    return in.substr(t2+1);
}

ostream& operator<< (ostream &s, CCoord C)
{
    return s << C.str();
}

int CParam::init_from_a01( const string in )
{
    int ji = in.find_first_not_of(" \t");
    int ret=-9;
    if( ji>=20 || ( ret = mystd::any2dbl( in.substr(ji,20), &val ) ) )
        return ret;
    Co.load( in.substr(21) );
    return 0;
}

string CParam::str(void) const
{
    return strg(val) + string(" ") + Co.str();
}
