//**************************************************************************//
//* FRIDA: fast reliable interactive data analysis                         *//
//* edif.cpp: edit on-line files (text_edit, dir, make, plot, ...)         *//
//* (C) Joachim Wuttke 1990-, v2(C++) 2001-                                *//
//* http://frida.sourceforge.net                                           *//
//**************************************************************************//

#include <stdlib.h>
#include <stdio.h>
#include <string>
#include <vector>
#include <algorithm>

#include "olm.h"
#include "asi.h"
#include "gar.h"
#include "edif.h"
#include "dualplot.h"

using namespace std;

namespace NEdif {
    CParam* RFind(CEle* e, CCoord OneR);
}


//! Display list of online files.

void NEdif::Dir(void)
{
    CEle *e;
    NOlm::IterateEle iter;
    iter.All();
    uint npts;
    string txtpts;
    while((e=iter())) {
        npts = e->nPts();
        if (npts) 
            txtpts = strg(npts);
        else
            txtpts = "?";

        char ztext[17];
        if( e->P()->nZ() ){
            if     ( e->P()->ZCo.size()==1 )
                sprintf( ztext, e->P()->ZCo[0].str().substr(0,16).c_str() );
            else if( e->P()->ZCo.size()==2 )
                sprintf( ztext, ( e->P()->ZCo[0].str().substr(0,10) + ", " +
                                  e->P()->ZCo[1].str().substr(0,4) ).c_str() );
            else if( e->P()->ZCo.size()==3 )
                sprintf( ztext, ( e->P()->ZCo[0].str().substr(0,5) + ", " +
                                  e->P()->ZCo[1].str().substr(0,4) + ", " +
                                  e->P()->ZCo[2].str().substr(0,3) ).c_str() );
            else
                sprintf( ztext, ( e->P()->ZCo[0].str().substr(0,4) + ", " +
                                  e->P()->ZCo[1].str().substr(0,3) + ", " +
                                  e->P()->ZCo[2].str().substr(0,2) + " &c" 
                             ).c_str() );
        } else {
            strcpy( ztext,"" );
        }
        printf("%3d%1s%-16s %5d %-16s %6s %-12s %-12s\n",
               iter.SelNo(),
               e->P()->as_on_disk ? "=" : " ",
               e->P()->name.substr(0,16).c_str(),
               e->nScan(), 
               ztext,
               txtpts.c_str(),
               e->P()->xco.str().substr(0,12).c_str(),
               e->P()->yco.str().substr(0,12).c_str()
            );
    }
}


//! Display list of z-values and x-ranges for scans in selected files.

void NEdif::DirZ(void)
{
    NOlm::IterateEle iter;
    CEle *e;
    COld *d;
    COlc *c;
    while((e=iter())) {
        cout << "file " << iter.SelNo() << ":\n";
        printf("   ");
        c = e->C();
        d = e->D();
        for (uint m=0; m<e->P()->ZCo.size(); m++) {
            printf(" %12s", e->P()->ZCo[m].str().c_str());
        }
        printf("%8s %38s\n", "#pts.", "x");
        for (uint j=0; j<e->nScan(); j++) {
            printf("%3u", j);
            for (uint m=0; m<e->P()->ZCo.size(); m++) {
                printf(" %12.6g", (*(e->Z(j)))[m]);
            }
            if (d)
                printf("%8u %.38s\n", d->VS[j].size(),
                       d->VS[j].x.str().c_str());
            else
                printf("\n");
        }
    }
}


//! Display z coordinates (names and units) for selected files.

void NEdif::ListZ(void)
{
    NOlm::IterateEle eiter;
    CEle *e;
    uint k, nz, i;
    string act;
    vector<CCoord> ZCo;

    if (!(e=eiter())) return;

    // Get current coordinates:
    ZCo = e->P()->ZCo;
    nz = ZCo.size();
    if (nz) 
        for (i=0; i<nz; ++i)
            printf("  z%u: %s\n", i, ZCo[i].str().c_str());
    else
        printf("  there are no z coordinates\n");
       
    // Check other input files:
    while((e=eiter())) {
        k = eiter.SelNo();
        VTEST ((e->P()->ZCo.size()!=nz), \
               ("! file %u has different number of z coordinates\n", k));

        for (i=0; i<nz; ++i)
            VTEST ((e->P()->ZCo[i]!=ZCo[i]), \
                   ("! file %u has different z coordinate %u\n", k, i));
    }

}


//! Display numeric parameters for selected files.

void NEdif::DirNumpar(void)
{
    NOlm::IterateEle iter;
    CEle *e;
    COld *d;
    COlc *c;
    while((e=iter())) {
        cout << "file " << iter.SelNo() << ":\n";
        printf("   ");
        for (uint m=0; m<e->P()->RPar.size(); m++) {
            cout << "  " << e->P()->RPar[m].str() << "\n" ;
        }
    }
}


//! Interactive modification of numeric parameters.

//! QUESTIONABLE. Why not use generic expressions and specific commands?

void NEdif::EditNumpar(void)
{
    NOlm::IterateEle eiter;
    CEle *e;
    vector<CCoord> AllR;
    CParam *rp;
    vector<uint> RStat; // 1: different values 2: partially undefined
    vector<double> RVal;
    string choice;
    IndexSet Choice;
    IndexSetIterator ISet;
    CCoord newco;
    double val;
    uint i, ii;
    bool first;

 make_table:

    // Collate RPar entries:
    AllR.clear();
    eiter.Reset();
    while((e=eiter())) {
        for (i=0; i<e->P()->RPar.size(); ++i) {
            for (ii=0; ii<AllR.size(); ++ii)
                if (e->P()->RPar[i].Co == AllR[ii])
                    goto found;
            AllR.push_back(e->P()->RPar[i].Co);
        found:
            ;
        }
    }
    RStat.reserve(AllR.size());
    RVal.reserve(AllR.size());
    for (ii=0; ii<AllR.size(); ++ii) {
        RStat[ii] = 0;
        first = 1;
        eiter.Reset();
        while((e=eiter())) {
            if ((rp=RFind(e, AllR[ii]))) {
                if (first) {
                    RVal[ii] = rp->val;
                    first = 0;
                } else if (RVal[ii] != rp->val) {
                    RStat[ii] |= 1;
                }
            } else {
                RStat[ii] |= 2;
            }
        }
    }
	
    // Show table:
    for (ii=0; ii<AllR.size(); ++ii) {
        printf("   %3u %.40s", ii, AllR[ii].str().c_str());
        if (RStat[ii] & 1)
            printf("<different values> ");
        else
            printf("%18.10g ", RVal[ii]);
        if (RStat[ii] & 2)
            printf("<partially undefined>\n");
        else
            printf("\n");
    }
		
 user_choice:
    choice = wask("Modify r-pars (h=help) ?", 3, "q");
    if (choice == "h") {
        printf("Enter either list of r-par numbers"
               " or a:add\n");
        goto user_choice;
    } else if (choice == "a") {
        newco.Ask("New parameter");
        val = dask("Value");
        eiter.Reset();
        while((e=eiter()))
            e->P()->RPar.push_back(CParam(newco, val));
        goto make_table;
    } else if (choice=="" || choice=="q") {
        return;
    } else if (Choice.parse(choice, 0, AllR.size())) {
        mystd::bell();
        goto user_choice;
    } 

    // It's a list

    // User action on list of parameters:
    ISet.reset(Choice);
    while (ISet(&ii)) {
        if (RStat[ii]) {
            // Table:
/*
  eiter.Reset();
  while((e=eiter())) {
*/					
        }
    action_on_par:
        choice = wask(("Action on r-par[" + strg(ii) +"] <val,d,q,<>>"
                          ).c_str());
        if        (choice=="h") {
            printf(" q     quit\n"
                   " d     delete entry\n"
                   " <val> set uniform value\n"
                   " f     set value per file\n");
            goto action_on_par;
        } else if (choice=="q") {
            return;
        } else if (choice=="") {
        } else if (choice=="d") {
/*				eiter.Reset();
				while((e=eiter())) {
				
				for (i=0; i<e->P()->RPar.size(); ++i)
                                if (e->P()->RPar[i] 
                                == AllR[ii]) {
                                e->P()->RPar.erase(
						
                                printf("missing ...\n");
*/
        } else if (sscanf(choice.c_str(), "%lg", &val)==1) {
            eiter.Reset();
            while((e=eiter())) {
                if ((rp=RFind(e, AllR[ii])))
                    rp->val = val;
                else 
                    e->P()->RPar.push_back(
                        CParam(AllR[ii], val));
            }
        } else {
            mystd::bell();
            goto action_on_par;
        } // action on rpar
    } // list of rpars

    goto make_table;
}


//! Auxiliary routine for EditNumpar.
	
CParam* NEdif::RFind(CEle* e, CCoord OneR)
{
    for (uint i=0; i<e->P()->RPar.size(); ++i)
        if (e->P()->RPar[i].Co == OneR) return &(e->P()->RPar[i]);
    return 0;
}


//! Display coordinate names and units for selected files.

void NEdif::DirCoord(void)
{
    NOlm::IterateEle eiter;
    CEle *e;
    string out;
    char   aux[4];
    while((e=eiter())) {
        sprintf( aux, "%3d", eiter.SelNo() );
        out = string( aux );
        out += " x: " + e->P()->xco.str();
        out += "  y: " + e->P()->yco.str();
        for (uint i=0; i<e->P()->nZ(); i++) {
            out += "  z" + strg(i) + ": " + e->P()->ZCo[i].str() + "\n";
        }
        cout << out;
    }
}


//! Modify coordinate names and units.

void NEdif::EditCoord( string which )
{
    NOlm::IterateEle eiter;
    CEle *ein, eout;
    COlo *fin, *fout;
    CRef cref( which );
    if ( !cref.defined() ) {
        printf( "! invalid coordinate %s\n", which.c_str() );
        return;
    }
    CCoord new_co, old_co;
    new_co.Ask( (which+" coordinate").c_str() );
    if (!NOlm::overwrite) NOlm::SelNew();
    while( (ein=eiter()) ) {
        fin = ein->P();
        if( !(NOlm::overwrite) ){ 
            NOlm::OloDup( fin, &eout );
            fout = eout.P();
            *fout = *fin;
        } else {
            fout = fin;
        }
                              
        if        ( cref.var==CRef::_X ) {
            old_co = fin->xco;
            fout->xco = new_co;
        } else if ( cref.var==CRef::_Y ) {
            old_co = fin->yco;
            fout->yco = new_co;
        } else if ( cref.var==CRef::_Z ) {
            if( cref.num >= fin->nZ() ){
                printf( "! not all files in selection have %s, use oz+\n",
                        which.c_str() );
                return;
            }
            old_co = fin->ZCo[cref.num];
            fout->ZCo[cref.num] = new_co;
        }
        fout->lDoc.push_back( "coordinate "+string(which)+" renamed from "+
                              old_co.str()+" into "+new_co.str() );
        if(!(NOlm::overwrite)) NOlm::OloPushBack( &eout );
    }
}


//! Display documentation lines for selected files.

//! MISSING: Interactive modification of documentation.

void NEdif::EditDoc(void)
{
    NOlm::IterateD fiter;
    COld *f;
    uint i;
    while((f=fiter())) {
        cout << "file " << fiter.SelNo() << " has doc-lines:\n";
        for (i=0; i<f->lDoc.size(); i++)
            cout << f->lDoc[i] << "\n";
    }
}

void NEdif::Save( string fmt )
{
    COlo *f;
    string fnam;
    FILE *fd;
    int err;

    CEle *e;
    NOlm::IterateEle iter;
    while((e=iter())) {

        if ( !(f = e->P()) ) {
            printf("SEVERE / f ?\n");
            return;
        }
        string quest = "Save as (."+fmt+")";
        fnam = wask( quest.c_str(), f->name);
        if ( fnam==string("") ) return;
        if ( fmt=="a01" )
            f->name = fnam;
        fnam += "."+fmt;

        if ( (fd = fopen(fnam.c_str(), "r")) ) {
            fclose(fd);
            if (!bask("Overwrite ?")) return;
        }
        if ( !(fd = fopen(fnam.c_str(), "w")) ) {
            printf("! cannot write to file %s\n", fnam.c_str());
            return;
        }

        err = e->Save( fd, fmt );

        fclose(fd);

        if (err) return;
    }
}

void NEdif::ReadIn(void)
{
    COld olf;
    static CCoord xco, yco, zco;
	
    static string fnam;
    olf.name = wask("Save as", fnam);

    // *** set coordinates ***
    olf.xco = xco.Ask("x", ASK_DEF_GIVEN, xco);
    olf.yco = yco.Ask("y", ASK_DEF_GIVEN, yco);
    cout << "if there is only one scan, answer \"\\\" to the following:\n";
    zco.Ask("z", ASK_DEF_GIVEN, zco);
    if(zco.defined()) olf.ZCo.push_back(zco);

    // *** set input mode ***
    cout << "input modes:\n"
        "  (1) enter x pointwise\n"
        "  (2) set regular x values\n"
        "  (3) enter x-y pairs\n"
        "  (4) select x,y from multicolumn input\n"
        "  (5) select x,y,z from multicolumn input\n";
    static int mInpX = -1;
    mInpX = iask_lim("Option", mInpX, CLimits(0,5));
    if( mInpX==5 && !zco.defined() ){
        printf( "need z coordinate for this option\n" );
        return;
    }

    static int imcx, imcy, imcz; 
    int mInpY=-1; // ask
	
    switch(mInpX) {
    case 0: 
        return;
    case 1: case 2:
        break;
    case 3:
        imcx = 0;
        imcy = 1;
        mInpY = 0;
        break;
    case 4: case 5:
        imcx = iask("x from column #", imcx);
        if (imcx<0) return;
        imcy = iask("y from column #", imcy);
        if (imcy<0) return;
        mInpY = 0;
        break;
    default:
        cout << "PROGRAM ERROR in Make: invalid switch(mInpX)\n";
        return;
    }
			
    if (mInpY<0) {
        cout << "input modes:\n"
            "  (1) enter y pointwise\n"
            "  (2) set regular y values\n"
            "  (3) select y from multicolumn input\n";
        static int mInpY_in = -1;
        mInpY_in = iask_lim("Option", mInpY_in, CLimits(0,3));
        if (!mInpY) return;
        mInpY = mInpY_in;
    }

    switch(mInpY) {
    case 0: case 1: case 2:
        break;
    case 3:
        imcy = iask("y from column #", imcy);
        if (imcy<0) return;
        break;
    default:
        cout << "PROGRAM ERROR in Make: invalid switch(mInpY)\n";
        return;
    }

    static int mInpZ;
    if        ( mInpX==5 ){
        mInpZ = 3;
    } else if (zco.defined()) {
        cout << "at present, we allow only _one_ z value per scan\n";
        cout << "input modes:\n"
            "  (1) enter z per spectrum\n"
            "  (2) select z from multi-z header line\n";
        mInpZ = iask_lim("Option", mInpZ, CLimits(0,2));
        if (!mInpZ) return;
    } else {
        mInpZ = 0;
    }

    switch (mInpZ) {
    case 0: case 1:
        break;
    case 2:
        if (! (mInpX==4 || mInpY==3)) {
            cout << "invalid combination of options:"
                " y is not read from multicolumn format\n";
            return;
        }
        break;
    case 3:
        imcz = iask( "z0 from column", imcz );
        if( imcz<0 ) return;
        break;
    default:
        cout << "PROGRAM ERROR in Make: invalid switch(mInpZ)\n";
        return;
    }

    static int imcy_delta=0; 
    static uint imcy_last;
    cout << "DEBUG "<< mInpZ << "\n";
    if (mInpZ==2) {
        imcy_delta = iask("step in number of y column", imcy_delta);
        if (imcy_delta) 
            imcy_last = iask("last y column", imcy_last);
    } else {
        imcy_delta = 0;
    }

    CScan S;
    static CGrid xgrid, ygrid;
    uint n = 0;
    uint imcm;
    string quest, line;
    double val, zold;
    vector<double> linv;

    // *** loop over scans ***
    int ns = 0;
    while(1) {

        S.Clear();

        switch(mInpZ) {
        case 1:
            line = sask("Enter z value", ASK_DEF_GIVEN|ASK_DEF_NOT_SHOW, "");
            // cout << "DEBUG: line [" << line << "]\n";
            if (mystd::str2vec(line, &linv, 2) < 0) {
                cout << "-> invalid line [" << line << "]\n";
                goto store;
            }
            // printf ("DEBUG: size = %d\n", linv.size());	
            if (linv.size()<1) {
                cout << "found no z in line [" << line << "] => assuming end-of-input\n";
                goto store;
            }
            // printf ("DEBUG: val = %g\n", linv[0]);	
            val = linv[0];
            break;
        case 2:
            line = sask("Enter header line with z values",
                        ASK_DEF_GIVEN|ASK_DEF_NOT_SHOW, "");
            if (mystd::str2vec(line, &linv, imcy+1)) {
                cout << "-> bad input\n";
                goto store;
            }
            if (linv.size()<=imcy) {
                cout << "-> not enough entries in header\n";
                goto store;
            }
            val = linv[imcy];
            break;
        }
        if (mInpZ<=2)
            S.z.push_back(val);

        switch(mInpX) {
        case 1:
            cout << "Enter x values (or empty line to exit):\n";
            while (1) {
                quest = "[" + strg(n) + "] ";
                line = sask(quest.c_str(), ASK_DEF_GIVEN|ASK_DEF_NOT_SHOW, "");
                if (sscanf(line.c_str(), "%lg", &val)!=1) break;
                S.x.push_back(val);
                n++;
            }
            break;
        case 2:
            S.x.Ask("x");
            n = S.x.size();
            break;
        case 3: case 4: case 5:
            cout<<"Enter data row (or empty line to exit):\n";
            imcm = (imcx>imcy) ? imcx : imcy;
            if( mInpY==5 && imcz>imcm )
                imcm = imcz;
            while (1) {
                quest = "[" + strg(n) + "] ";
                line = sask(quest.c_str(), ASK_DEF_GIVEN|ASK_DEF_NOT_SHOW, "");
                if (line==string("")) break;
                if (mystd::str2vec(line, &linv, imcm+1)) {
                    printf(
                        " could not extract value no. %u"
                        " from string [%s]\n", imcm, line.c_str());
                    continue;
                }
                if (linv.size()<=imcm) {
                    cout << "-> not enough entries in line\n";
                    continue;
                }
                if( mInpZ==3 ){
                    if( n==0 ){
                        S.z.push_back(linv[imcz]);
                        zold = linv[imcz];
                    } else if( linv[imcz]!=zold ){
                        olf.VS.push_back(S);
                        ns++;
                        S.Clear();
                        S.z.push_back(linv[imcz]);
                        zold = linv[imcz];
                    }
                }
                S.push_back(linv[imcx], linv[imcy]);
                n++;
            }
            break;
        }
        if (n<=0) return;

        switch(mInpY) {
        case 0:
            break;
        case 1:
            cout << "Enter " << n << " y values:\n";
            S.y.resize(n);
            for (uint i=0; i<n; i++) {
                quest = "x[" + strg(i) + "] = " +
                    strg(S.x[i]) + " -> ";
                do {
                    line = sask(quest.c_str());
                } while (sscanf(line.c_str(), "%lg", &val)!=1);
                S.y[i] = val;
            }
            break;
        case 2:
            ygrid.Ask("y", n);
            ygrid.AsVec(&(S.y));
            break;
        case 3:
            cout << "Enter " << n << " lines with y values:\n";
            S.y.resize(n);
            for (uint i=0; i<n; i++) {
                quest = "x[" + strg(i) + "] =" +
                    strg(S.x[i]) + " -> ";
                while (1) {
                    line = sask(quest.c_str());
                    if (mystd::str2vec(line, 
                                       &linv, imcy+1)) {
                        printf(
                            " could not extract y value no. %u"
                            " from string [%s]\n", imcy, line.c_str());
                        continue;
                    }
                    if (linv.size()<=imcy) {
                        cout << "-> not enough entries"
                            " in line\n";
                        continue;
                    }
                    break;
                }
                S.y[i] = linv[imcy];
            }
            break;
        }
        olf.VS.push_back(S);
        ns++;
		
        if (!mInpZ || mInpZ==3) goto store;
        if (imcy_delta) {
            imcy += imcy_delta;
            if (imcy_delta>0 && imcy>imcy_last ||
                imcy_delta<0 && imcy<imcy_last) goto store;
        }
    }
	
    // *** store new on-line file ***
 store:
    // cout << "DEBUG: store\n";
    if (ns<=0) return;
    // cout << "DEBUG: sel\n";
    NOlm::SelNew();
    // cout << "DEBUG: add\n";
    NOlm::OloAdd(&olf);
    // cout << "DEBUG: main\n";
}

void NEdif::ReadTab( bool multiblock, bool fromscript )
{
    COld olf;
    NOlm::SelNew();
        
    static string script;
    string inFile, s1, s2;

    if( fromscript ){
        inFile = "/ramdisk/tab";
        cout << "Script must write to "<<inFile<<"\n";
        script  = wask("Script", script);
        system( script.c_str() );
    } else {
        inFile = wask("Read tab from file");
    }
    FILE *fd;
    if (!(fd = fopen(inFile.c_str(), "r"))) {
        printf ("! cannot open file %s\n", inFile.c_str());
        return;
    }

    mystd::fname_divide( inFile, 0, &(olf.name), 0 );
    olf.lDoc.push_back( "data from table "+inFile );

    olf.xco = CCoord("x", "");
    olf.yco = CCoord("y", "");

    string lin;
    vector<double> dat, zdat;
    int ni = 0;
    int nblock = 0;
    int nline = -1; // line in block
    int nz = 0;
    double val;
    CScan S;
    while( mystd::freadln(fd, &lin)>-1 ) {
        if( lin.substr(0,4)=="#rpa" ){
            mystd::string_extract_word( lin.substr(4), &s1, &s2 );
            if( mystd::any2dbl( s2, &val ) )
                printf( "invalid parameter line [%s]\n", lin.c_str() );
            else
                olf.RPar.push_back( CParam( s2, "", val) );
            continue;
        }
        if( lin[0]=='#' )
            continue;

        if( nline==-1 ) { 
            if ( multiblock ) {
                if( mystd::str2vec(lin, &zdat, 0, false) ) break;
                if( nz==0 ){ // first such header (must be first line of file)
                    nz = 1 + zdat.size();
                    for( uint iz=0; iz<nz-1; ++iz )
                        olf.ZCo.push_back( CCoord("z"+strg(iz), "") );
                    olf.ZCo.push_back( CCoord("line", "") );
                } else if( zdat.size()+1 != nz ){
                    printf("read2tab header in line %i has length %i instead"
                       " of %i-1", ni, zdat.size(), nz );
                    return;
                }
            } else if (nz==0) {
                nz = 1; 
                olf.ZCo.push_back( CCoord("line", "") );
           }
            nline = 0;
            ++nblock;
            if( multiblock )
                continue;
        }

        if( lin=="" ) {
            nline = -1;
            ++nblock;
            continue;

        }

        // regular line -> one scan
        S.Clear();
        S.z.resize( nz );
        for( uint iz=0; iz<nz-1; ++iz )
            S.z[iz] = zdat[iz];
        S.z[nz-1] = nline++;
        
        if( mystd::str2vec(lin, &dat, 0, false) ){
            printf("read2tab irregular line %i in block %i [%s]\n",
                   nline, nblock, lin.c_str());
            break;
        } else if(dat.size()==0){
            printf("read2tab empty line %i in block %i [%s]\n",
                   nline, nblock, lin.c_str());
            break;
        } 

        for( int i=0; i<dat.size(); ++i ){
            S.push_back( (double)i, dat[i] );
        }
        olf.VS.push_back(S);
    }

    NOlm::SelNew();
    NOlm::OloAdd(&olf);
}

void NEdif::MakeGrid(void)
{
    COld olf;

    static int ni=1, nj=1;
    static string fnam;

    int niIn = iask("Number of points per scan", ni);
    if (niIn<1) return;
    ni = niIn;

    int njIn = iask("Number of scans", nj);
    if (njIn<1) return;
    nj = njIn;

    fnam = "grid"+strg(ni);
    if (nj>1) fnam += "x"+strg(nj);

    olf.name = wask("Save as", fnam);

    // *** set coordinates ***
    olf.xco = CCoord("x", "");
    olf.yco = CCoord("y", "");
    if (nj>1)
        olf.ZCo.push_back(CCoord("z", ""));

    CScan S;

    // *** loop over scans ***
    for (int j=0; j<nj; ++j) {

        S.Clear();

        S.x = CGrid( ni, 0., 1. );
        S.y.clear();
        S.y.resize( ni, 0. );

        if (nj>1) S.z.push_back( (double)j );

        olf.VS.push_back(S);
    }
	
    NOlm::SelNew();
    NOlm::OloAdd(&olf);
}

void NEdif::Plot(bool add)
{
    COld *f;
    CList JSel;
    CScan *J;
	
    if (NOlm::FJSelAsk("Plot which scans")) return;

    if (!add) {

        // determine x-range ?
		
        if (!NPlot::X.R.finite()) {
            NPlot::X.R = CRange(2); // empty
            NOlm::IterateD fiter;
            while((f=fiter())) {
                NOlm::FJSelGetNoask(fiter.SelNo(), &JSel);
                IListV<CScan> Jiter(&(f->VS), &JSel);
                while ((J=Jiter())) 
                    NPlot::X.R *= J->x.range(NPlot::X.log);
            }
            if (NPlot::X.R.empty()) {
                cout << "x-range is empty\n";
                return;
            }
            NPlot::X.R.Round(0, .05, 2.);
            if (NPlot::X.R.empty()) {
                cout << "x-range emptied by round\n";
                return;
            }
        }

        // determine y-range ?
        if (!NPlot::Y.R.finite()) {
            NPlot::Y.R = CRange(2);
            NOlm::IterateD fiter;
            while((f=fiter())) {
                NOlm::FJSelGetNoask(fiter.SelNo(), &JSel);
                IListV<CScan> Jiter(&(f->VS), &JSel);
                while ((J=Jiter())) 
                    NPlot::Y.R *= J->get_yrange(
                        NPlot::X.R, NPlot::Y.log);
            }
            if (NPlot::Y.R.empty()) {
                cout << "y-range is empty\n";
                return;
            }
            NPlot::Y.R.Round(0, .05, 2.);
            if (NPlot::Y.R.empty()) {
                cout << "y-range emptied by round\n";
                return;
            }
        }

        if (!NPlot::X.R.finite() || !NPlot::Y.R.finite()) {
            cout << "could not determine data range "
                "(only curves ?)\n";
            return;
        }

        // build label: (zur Zeit nur vom ersten File; definiere +=)
        NPlot::X.C = NOlm::MOM[*(NOlm::FSel.V.begin())].P()->xco;
        NPlot::Y.C = NOlm::MOM[*(NOlm::FSel.V.begin())].P()->yco;

        // draw new frame:
        NPlot::Clear();
        NPlot::Box();
    }

    // plot data:
    // printf("DEBUG now plotting\n"); // Fehlersuche JSel / empty file
    CEle *e;
    uint k, j;
    COld *fd;
    COlc *fc;
    CScan S, *s;
    int lstyle;
    NOlm::IterateEle eiter;
    ListIter jiter;
    int npts = 1000;
    while((e=eiter())) {
        k = eiter.SelNo();
        NOlm::FJSelGet(eiter.SelNo(), &JSel);
        jiter.Reset(JSel);
        while(jiter(&j)) {
            if ((fd = e->D())) {
                s = &(fd->VS[j]);
                lstyle = 0;
            } else if ((fc = e->C())) {
                S.x = CGrid( npts, NPlot::X.R, NPlot::X.log ?
                             CGrid::_LOG : CGrid::_LIN );
                S.y.resize( npts );
                for ( uint i=0; i<npts; i++ )
                    S.y[i] = fc->T.tree_curve_val( S.x[i], fc, k, j, i );
                s = &S;
                lstyle = -1;
            } else {
                cout << "PROGRAM ERROR plot invalid *e\n";
                exit(7);
            }
            NPlot::Line(lstyle, s->x, s->y, e->Z(j),
                        e->P()->xco.str(), e->P()->yco.str());
        }
        NPlot::Doc(e->P()->lDoc);
        NPlot::Tekx();
    }
}
