#include <stdio.h>
#include <iostream>
#include <vector>

//! Context a generic expression is used in.

class CContext {
 public:
    static const int HAS_XARG = 1;
    inline CContext(
        class COlo *f_in=0, uint k_in=-1, uint j_in=-1, uint i_in=-1,
        uint flags_in=0, double xarg_in=0 )
        : f(f_in), k(k_in), j(j_in), i(i_in), flags(flags_in), xarg(xarg_in)
        {};
    COlo* f;
    uint k;
    uint j;
    uint i;
    uint flags;
    double xarg;
};


//! References to data points, indices, ...; used as endpoints of CTree's.

class CRef {
 public:
    enum TVar { _NOREF=0, _X, _Y, _Z, _FC, _FP, _FM, _K, _J, _I, 
                _NJ, _NI, _AK, _AJ, _AI };
    TVar var;
    uint num;
    class CTree *tk;
    class CTree *tj;
    class CTree *ti;
	
    CRef( const string s="", 
         class CTree* tk_in=0, class CTree* tj_in=0, class CTree* ti_in=0);
    ~CRef();

    CRef& operator=(const CRef&);

    inline bool defined() const {
        return var!=_NOREF; };
    inline bool isindex() const {
        return var==_K || var==_J || var==_I; };
    inline bool issize() const {
        return var==_NJ || var==_NI; };
    inline bool isask() const { 
        return var==_AK || var==_AJ || var==_AI; };
    inline bool pointwise() const { 
        return var==_X || var==_Y || var==_I || var==_AI; };
    inline bool scanwise() const {
        return pointwise() || var==_Z || var==_FP || var==_FM || var==_J ||
            var==_NI || var==_AJ || var==_FC; };
    inline bool unspecific() const { 
        return isindex() || issize() || isask(); };
    inline bool indatafile() const {
        return unspecific() || var==_X || var==_Y || var==_Z; };
    inline bool incurvefile() const {
        return (unspecific() && !pointwise())
            || var==_FC || var==_FP || var==_FM || var==_Z ; };
    inline bool numbered() const {
        return var==_FP || var==_FM || var==_Z || isask(); };
    inline bool assignable() const {
        return defined() && !unspecific(); };
    inline bool indexed() const {
        return ti || tj || tk; };
    
    int ref_val( void *ret, CContext *ctx ) const;
    int coord( CCoord *ret, class COlo* f, uint k ) const;
    string ref_info() const;
    inline const char* ref_cstr() const {
        return ref_info().c_str(); };
};


//! Vector of CTrees, currently only used in 'a in [1,2,4]' construct.

typedef vector<CTree*> VCTree;


//! Parsed generic expression.

class CTree {
 private:
    enum TreeTyp { _NOTREE=0, _MONOP, _BINOP, _TEROP, _OP_T_V,
                   _VAL, _REF, _FRF };
    TreeTyp typ;
    double val;
    CFunc* fun;
    CTree* arg1;
    CTree* arg2;
    VCTree* varg2;
    CTree* arg3;
    CRef*  ref;

 public:
    inline CTree()
        : typ(_NOTREE), fun(0) {};
    inline CTree( double v_in )
        : typ(_VAL), val(v_in), fun(0) {};
    inline CTree( CFunc *f_in, CTree *a1 )
        : typ(_MONOP), fun(f_in), arg1(a1) {
        if (fun->narg!=1) {
            cout << "PROG ERR: CTree(1) with conflicting #arg\n";
            typ=_NOTREE;
            return;
        }
    };
    inline CTree( CFunc *f_in, CTree *a1, CTree *a2 )
        : typ(_BINOP), fun(f_in), arg1(a1), arg2(a2) {
        if (fun->narg!=2) {
            cout << "PROG ERR: CTree(2) with conflicting #arg\n";
            typ=_NOTREE;
            return;
        }
    };
    inline CTree( CFunc *f_in, CTree *a1, CTree *a2, CTree *a3 )
        : typ(_TEROP), fun(f_in), arg1(a1), arg2(a2), arg3(a3) {
        if (fun->narg!=3) {
            cout << "PROG ERR: CTree(3) with conflicting #arg\n";
            typ=_NOTREE;
            return;
        }
    };
    inline CTree( CFunc *f_in, CTree *a1, VCTree *va2 )
        : typ(_OP_T_V), fun(f_in), arg1(a1), varg2(va2) {
        if ( !fun ) {
            cout << "PROG ERR: Function not found\n";
            typ=_NOTREE;
            return;
        } else if (fun->narg!=2) {
            cout << "PROG ERR: CTree(2) with conflicting #arg\n";
            typ=_NOTREE;
            return;
        }
    };
    inline CTree( CRef* ref_in )
        : typ(_REF), fun(0), ref(ref_in) {};
    inline CTree( CRef* ref_in, CTree* a1 )
        : typ(_FRF), fun(0), ref(ref_in), arg1(a1) {};
    inline ~CTree() {
        switch(typ) {
        case _TEROP:
            delete arg3;
        case _BINOP:
            delete arg2;
        case _MONOP:
            delete arg1;
            break;
        case _REF:
            delete ref;
        default:
            ;
        }
    };

    CTree& operator=( const CTree& );
    int tree_val( double *ret, CContext *ctx ) const;
    int tree_val(
        double *ret, class COlo* f=0, 
        uint k=(uint)-1, uint j=(uint)-1, uint i=(uint)-1,
        uint flags=0, double xarg=0.0 ) const;
    int tree_uival( uint *ret, CContext *ctx ) const;
    double tree_curve_val( 
        double xarg, class COlc* fc, uint k, uint j, uint i ) const;
    int coord( CCoord *ret, COlo *f=0, uint k=(uint)-1 ) const;
    uint npar( CRef::TVar r ) const;
    string tree_info() const;
    inline const char* tree_cstr() const {
        return tree_info().c_str(); };

 private:
    void nparx( CRef::TVar v, uint *np ) const;
};


//! Direct access from main loop.

namespace NCalc {
    void Calculator();
};
