#include<vector>

class CGrid {
 public:
	enum GridTyp { _NOGRID=0, _LIN, _LOG, _VEC };
 private:
	GridTyp kind;
	double gi, gf;
	uint ng;
	vector<double> v;
 public:
	CGrid(void);
	CGrid(const vector<double>&);
	CGrid(const double*, const size_t);
	CGrid(const uint n_in, double start=0., double step=0.);
	CGrid(const uint n_in, const CRange& R_in, GridTyp k_in=_LIN);

	void simplify(void);
	void try_lin(void);
	void try_log(void);
	void set_vec(void);

	CGrid Ask(const string& vnam, const uint n_in=0);
	void clear(void);
	void push_back(const double);
	void erase (const uint begin, const uint end);

	uint size(void) const;
	double operator[](const uint) const;
	uint AsVec(vector<double> *) const;
	uint AsVec(double *, size_t) const;
	CRange range(const bool logflag=0) const;
	inline double inf(void) const { return range().inf; };
	inline double sup(void) const { return range().sup; };
	int lin(double* rd=0, double* ri=0, double* rf=0);
	string kind_str(void) const;

	string str(void) const;
	inline const char* c_str(void) const { return str().c_str(); };
	int pos(double) const;
	uint pos_force(double) const;

	bool compatible(const CGrid& G2, string *err=0) const;
 private:
	bool eq(const double r1, const double r2, const double step) const;
	static const double grid_tol;
};



