//**************************************************************************//
//* FRIDA: fast reliable interactive data analysis                         *//
//* list.cpp                                                               *//
//* (C) Joachim Wuttke 1990-, v2(C++) 2001-                                *//
//* http://frida.sourceforge.net                                           *//
//**************************************************************************//

#include <math.h>
#include <stdio.h>
#include <iostream>
#include <stdlib.h>

#include<string>
#include<vector>
#include<algorithm>
using namespace std;

#include "mystd.h"
#include "range.h"
#include "gar.h"

#include "list.h"

#define ERRLII cout << "! invalid input: in list [" << instruction << "] "

CList::CList(string inp, CLimits Lims_in) 
{
	Lims = Lims_in;
	if (!Lims.test()) {
		printf ("!!! initializing CList(%s) with invalid limits %s\n", 
			inp.c_str(), Lims_in.str().c_str());
		instruction = "-";
		synval = 1; // ungerne: um Endlosschleife zu verhindern
		limneeded = 0;
		Lims = CLimits(1);
		return;
	}
	instruction = inp;
	Parse(inp);
}

CList::CList(uint val) 
{
	instruction = "";
	synval = 1;
	limneeded = 0;
	Lims = CLimits(1);
	V.push_back(val);
}

CList::CList(uint ni, uint nf) 
{
	if (nf<ni || nf==(uint)-1) {
		instruction = "-";
		synval = 1; // ungerne: um Endlosschleife zu verhindern
		limneeded = 0;
		Lims = CLimits(1);
		return;
	}

	instruction = "";
	synval = 1;
	limneeded = 0;
	Lims = CLimits(1);
	for (uint val = ni; val <= nf; ++val) {
		V.push_back(val);
	}
}

void CList::Parse(string inp)
{
	instruction = inp;
	synval = 1;
	limneeded = 0;
	V.clear();
	ParseList(instruction);
}

void CList::ParseList(string inp)
{
	if(inp==string("-")) return; // valid empty list
	string::size_type ji=0, jf;
	while (ji<inp.size()) {
		if ((jf=inp.find(',', ji)) == string::npos) jf = inp.size();
		ParseSkipinterval(inp.substr(ji,jf-ji));
		if(!semvalid()) return;
		ji = jf+1;
	}
}

void CList::ParseSkipinterval(string inp)
{
	// cout << inp << endl;
	int skip;
	string::size_type j;
	if ((j=inp.find('.')) != string::npos) {
		skip = ParseSkipfactor(inp.substr(j+1));
		if(!semvalid()) return;
	} else {
		skip = 1;
		j = inp.size();
	}
	ParseInterval(inp.substr(0,j), skip);
	if(!synval) return;
}

int CList::ParseSkipfactor(string inp)
{
	int val;
	char dummy;
	if (sscanf(inp.c_str(), "%d%c", &val, &dummy)!=1 || val<=0) {
		ERRLII "invalid skip factor [" << inp << "]\n";
		synval = 0;
		return 0;
	}
	return val;
}

void CList::ParseInterval(string inp, int skip)
{
	// cout << "intv: " << inp << endl;
	string::size_type j;
	uint vi, vf;
	bool xv=0; // vf exists ?
	if (inp==string("*")) {
		limneeded = 1;
		if (!limgiven()) return;
		vi = Lims.inf;
		vf = Lims.sup;
		xv = 1;
	} else if ((j=inp.find('-')) != string::npos) {
		vf = ParseListvalue(inp.substr(j+1),+1);
		if(!semvalid()) return;
		vi = ParseListvalue(inp.substr(0,j),-1);
		if(!semvalid()) return;
		xv = 1;
	} else {
		vi = ParseListvalue(inp,0);
		if(!semvalid()) return;
		vf = vi;
	}
	if(vi>vf) {
		ERRLII "invalid interval [" << inp << "]\n";
		synval = 0;
		return;
	}
	for (uint vv=vi; vv<=vf; vv += skip) {
		// cout << "push " << vv << endl; // DEBUG
		V.push_back(vv);
	}
}	

int CList::ParseListvalue(string inp, int which)
{
	int val;
	char dummy;
	if (inp==string("*")) {
		limneeded = 1;
		if (!limgiven()) return 0;
		if      (which==-1) 
			val = Lims.inf;
		else if (which==+1) 
			val = Lims.sup;
		else {
			ERRLII "list value * out of coninstruction\n";
			synval = 0;
			return 0;
		}
	} else if (sscanf(inp.c_str(), "%d%c", &val, &dummy)!=1 || val<0) {
		ERRLII "invalid list value [" << inp << "]\n";
		synval = 0;
		return 0;
	}
	if (!Lims.contained(val)) {
		ERRLII "list value [" << inp << "] out of bounds " 
				      << Lims << " \n";
		synval = 0;
		return 0;
	}
	return val;
}

string CList::str(void) const
{ 
	if (!synvalid()) return "<invalid list>";
	if (instruction != "") return instruction;
	return sel2text();
}

string CList::sel2text(void) const
{
	if(!V.size()) {
		return "-";
	}
	string s;
        vector<uint> VV=V;
	vector<uint>::iterator iV=VV.begin();
	uint vi, vd;
	while(iV<VV.end()) {
		s += strg(vi=*(iV++));
		if(iV>=VV.end()) break;
		vd = *(iV++) - vi;
		if(iV>=VV.end()) {
			s += "," + strg(vi+vd);
			break;
		}
		if (*iV!=vi+2*vd) {
			s += ",";
			iV--;
			continue;
		}
		while(iV<VV.end() && *(iV-1)+vd==*iV) iV++;
		s += "-" + strg(*(iV-1));
		if (vd!=1) s += "." + strg(vd);
		if(iV>=VV.end()) break;
		s += ",";
	}
	return s;
}

const char * CList::helptext = 
	 "a list:\n"
         "    list:            <empty_list> | <finite_list> | <infinite_list>\n"
         "    empty_list:      -\n"
         "    finite_list:     <range_with_skip> { , <range_with_skip> }\n"
         "    infinite_list:   *\n"  
         "    range_with_skip: <range> [ . <skip> ]\n"
         "    range:           <bound> [ - <bound>]\n"
	 "    skip:            (integer >= 1)\n"
	 "    bound:           * | (integer >= 0)\n";

char ListDecode(const char *resp, CList *rval, const CAskPars *AP)
{
	if (resp[0]=='\0') {
		return 'd';
	} else if (!strcmp(resp, "?")) {
		return 'h';
	}
	CLimits Lims;
	if (AP) {
		Lims = *((CLimits *) AP);
	} else {
		Lims = CLimits(1);
	}
	*rval = CList(string(resp), Lims);
	if (rval->synvalid())
		return 'v';
	else
		return 'h';
}

bool CList::contained(uint val) const
{
	for (uint i=0; i<V.size(); i++)
		if (val==V[i]) return 1;
	return 0;
}

void CList::ProvideLims(class CLimits Lims_in)
{
	if (!Lims_in.finite()) {
		printf("!! provided infinite limits to list\n");
		return;
	}
	Lims = Lims_in;
	Parse(instruction);
}

CList CList::Ask(const char* quest)
{
	return Ask(quest, 0, CList(), CLimits(1));
}

CList CList::Ask(const char* quest, CLimits Lims)
{
	CList Dummy;
	return Ask(quest, 0, CList(), Lims);
}

CList CList::Ask(const char* quest, CList Def)
{
	return Ask(quest, 1, Def, CLimits(1));
}

CList CList::Ask(const char* quest, CList Def, CLimits Lims)
{
	return Ask(quest, 1, Def, Lims);
}

void CList::Sort(void)
{
	sort(V.begin(), V.end());
	instruction = "";
}

void CList::SortReverse(void)
{
	Sort();
	reverse(V.begin(), V.end());
	instruction = "";
}

bool list_lim_check (const void *val, const void *args)
{
	CLimits *lim;
	lim = (CLimits *) args;
	CList *lis;
	lis = (CList *) val;
	for (uint i=0; i<lis->n(); i++)
		if (!(lim->contained((double) lis->V[i])))
			return false;
	return true;
}

CList CList::Ask(const char* quest, bool defflag, CList Def, CLimits Lims)
{
	VConstraints VC;
	string s;  
	s = "list values must be within limits " + Lims.str();
	VC.push_back(CConstraints(list_lim_check, (void *) &Lims, s));
	*this =  myask(string(quest), defflag | ASK_WORD, Def, 
		       helptext, ListDecode, (void *)&Lims, &VC);
	return *this;
}

CList CList::operator+(const CList L) const
{
	CList Z;
	return Z += L;
}

CList CList::operator+(const uint val) const
{
	CList Z;
	return Z += val;
}

CList CList::operator+=(const CList L)
{
	for (uint i=0; i<L.V.size(); ++i)
		*this += L.V[i];
	return *this;
}

CList CList::operator+=(const uint val)
{
	instruction = "";
	if (!contained(val)) V.push_back(val);
	return *this;
}





