//**************************************************************************//
//* FRIDA: fast reliable interactive data analysis                         *//
//* plotaux.cpp: ticks and tacks                                           *//
//* (C) Joachim Wuttke 1990-, v2(C++) 2001-                                *//
//* http://frida.sourceforge.net                                           *//
//**************************************************************************//

#include <stdlib.h>
#include <stdio.h> // for debug
#include <math.h>

#include "plotaux.h"

int plotaux::calc_ticks(int logscale, double vmin, double vmax, 
			int *ntack, double **tack, int *ntpt, double *ticklim)
{
    int ir, nt, i;
    double r, rd, rdr, d, vsub, vsup, tack0, tackn, dtack;
	
    if (vmin >= vmax) {
        printf("PROGRAM ERROR/ Ticks/ Bad Limits: %g %g\n",
               vmin, vmax);
        return -1;
    }

    if (!logscale) {
        r = log10(vmax - vmin);
        ir = (int) ((r<0) ? r-1 : r);
        rd = r - ir; // fractional part of r
        rdr = pow(10., rd);
        if        (rdr > 9.99999) {
            dtack = 2.5 * pow(10., ir); // spacing between tacks
            *ntpt = 5;                // ticks per tack
        } else if (rdr > 5.00001) {
            dtack = 2 * pow(10., ir);
            *ntpt = 4;
        } else if (rdr > 2.500005) {
            dtack = 1 * pow(10., ir);
            *ntpt = 5;
        } else if (rdr > 1.600003) {
            dtack = .5 * pow(10., ir);
            *ntpt = 5;
        } else if (rdr > 1.250002) {
            dtack = .4 * pow(10., ir);
            *ntpt = 4;
        } else {
            dtack = .25 * pow(10., ir);
            *ntpt = 5;
        }
        d = vmin / dtack;
        tack0 = dtack * (int) (d<=0?d-1:d);
        d = vmax / dtack;
        tackn = dtack * ((int) (d<0?d-1:d) + 1);
        vsub = vmin - (vmax-vmin)*1.e-4;
        vsup = vmax + (vmax-vmin)*1.e-4;
        nt = (int) ((tackn-tack0) / dtack) + 3; // allocate enough
        *tack = (double*) malloc(sizeof(double)*nt);
        *ntack = 0;
        for (i=0; i<nt; i++) {
            d = tack0 + i*dtack;
            if (vsub<=d && d<=vsup)
                (*tack)[(*ntack)++] = d;
        }

        ticklim[0] = tack0;
        ticklim[1] = (*tack)[(*ntack)-1] + dtack;
        // printf("DEBUG ticklim %g %g\n", ticklim[0], ticklim[1]);

        /* 
           nt = *ntpt * (*ntack+2);
           dtick = dtack / ntpt;
           *tick = (double*) malloc(sizeof(double)*nt);
           *ntick = 0;
           for (i=0; i<nt; i++) {
           d = (*tack)[0]-dtack + i*dtick;
           if (vsub<=d && d<=vsup)
           (*tick)[(*ntick)++] = d;
           }
        */
		
    } else { // log scale

        static double eins = 1.0000009999999999;
        double rlgmin, rlgmax, rlgrel, d1, d2, r0; 
        int    minexp, maxexp, incr, i1, i2;

        if (vmin <= 0) { 
            printf("PROGRAM ERROR/ Ticks/ negative log argument\n");
            return -1;
        }

        rlgmin = log10(vmin);
        rlgmax = log10(vmax);
        rlgrel = rlgmax - rlgmin;
        if (rlgrel > 40) {
            printf("!! Excessive log range\n");
            return -1;
        }

        minexp = (rlgmin>0 ? 
                  (int) (rlgmin+1e-6) :
                  ((int) (rlgmin-1e-6) -1));
        maxexp = (rlgmax>0 ? 
                  (int) (rlgmax+1e-6) :
                  ((int) (rlgmax-1e-6) -1));
        /* printf("DEBUG inf %g -> %d sup %g -> %d\n", 
           rlgmin, minexp, rlgmax, maxexp); */

        incr = (int) (rlgrel / 7 + 1); // #decades per tack
        if (incr==2) incr = 1; // preferences: 10^1, 10^3, 10^6 etc.
        if (incr>3) incr = 3 * (incr % 3);

        if (incr > 1) {
            minexp -= minexp % incr;
            maxexp -= maxexp % incr;
        }

        /*  - Set large ticks array : */
        static int mt=30;
        *tack = (double*) malloc(sizeof(double)*mt);

        *ntack = 0;
        i1 = maxexp;
        i2 = incr;
        d1 = vmin / eins;
        d2 = vmax * eins;
        for (i = minexp; i2 < 0 ? i >= i1 : i <= i1; i += i2) {
            r0 = pow(10., i);
            if (d1<=r0 && r0<=d2) {
                if(*ntack>=mt) {
                    printf("!!! PROG ERR too many tacks\n");
                    return -1;
                }
                (*tack)[(*ntack)++] = r0;
            }
        }

        if (*ntack >= 1) {
            ticklim[0] = (*tack)[0]          / pow(10., incr);
            ticklim[1] = (*tack)[(*ntack)-1] * pow(10., incr);
            /* printf("DEB %d TA %g %g LI %g %g FA %g\n",
             *ntack, (*tack)[0], (*tack)[(*ntack-1)],
             ticklim[0], ticklim[1], pow(10., incr)); */
        } else {
            printf("!! plotaux::calc_ticks not implemented:"
                   "tacks when no 10^n in range\n");
        }

        /** determine # ticks / tack **/
        if        (incr==1 && rlgrel<=7) {
            *ntpt = 9;
        } else if (incr==1 && rlgrel<=7) {
            *ntpt = 3; // 1-2-5-10 - Schritte 
        } else if (incr==1) {
            *ntpt = 1;
        } else {
            *ntpt = -incr; 
            // means incr ticks per tack
            // means 1 tick per decade
        }
    }

    return 0;
}
