//**************************************************************************//
//* FRIDA: fast reliable interactive data analysis                         *//
//* rng.cpp: random number generator wrapper                               *//
//* (C) Joachim Wuttke 1990-, v2(C++) 2001-                                *//
//* http://frida.sourceforge.net                                           *//
//**************************************************************************//

#include "rng.h"
#include <gsl/gsl_rng.h>
#include <gsl/gsl_randist.h>
#include <math.h>

namespace NRNG { // local declarations (global declarations are in rng.h)
        gsl_rng *r; // static variable
}

void NRNG::init() 
{
        gsl_rng_env_setup();
        NRNG::r = gsl_rng_alloc( gsl_rng_default );
}

double NRNG::uniform()
{
        return gsl_rng_uniform( NRNG::r );
}

double NRNG::uniform( const double rmin, const double rmax )
{
        return rmin + (rmax-rmin)*gsl_rng_uniform( NRNG::r );
}

double NRNG::gaussian( const double stdv, const double cutoff )
{
    double ret;
    do {
        ret = gsl_ran_gaussian ( NRNG::r, stdv );
    } while( fabs(ret)>cutoff );
    return ret;
}
