/* works with flex 2.5.4 */

%option noyywrap

%{
using namespace std;
#include <math.h>
#include <string>
#include <vector>
#include "olm.h"
#include "xax_yacc.h"

#define YY_DECL int xaxlex(YYSTYPE *xaxlval)
%}

DIG      [0-9]
ID       [a-zA-Z][a-zA-Z0-9]*
EXP      [eE]"-"?[0-9]+
%%

{DIG}+	{
	// printf( "lex: An integer: %s (%d)\n", xaxtext, atoi( xaxtext ) );
	xaxlval->val = atoi(xaxtext);
	return NUM; }

({DIG}+"."{DIG}*{EXP}?)|("."?{DIG}*{EXP}?) {
	// printf( "lex: A float: %s (%g)\n", xaxtext, atof( xaxtext ) );
	xaxlval->val = atof(xaxtext);
	return NUM; }

[xykji]|n[ij]|z{DIG}+|p{DIG}+|fm{DIG}+ {     
        // printf( "lex: The variable %s\n", xaxtext );
        xaxlval->ptr = (void*) xaxtext; 
        return REF; }

f {     
        // printf( "lex: The variable %s\n", xaxtext );
        xaxlval->ptr = (void*) xaxtext; 
        return FRF; }

pi {
	// printf( "lex: pi\n" );
	xaxlval->val = acos(-1.);
	return NUM; }

e {
	// printf( "lex: e\n" );
	xaxlval->val = exp(1.);
	return NUM; }

"=="  { return BOP_EQ; }
"!="  { return BOP_NE; }
"<"   { return BOP_LT; }
"<="  { return BOP_LE; }
">"   { return BOP_GT; }
">="  { return BOP_GE; }
"&&"  { return BOP_AND; }
"||"  { return BOP_OR; }
"xor" { return BOP_XOR; }
"mod" { return BOP_MOD; } /* man 3 fmod */
"div" { return BOP_DIV; } /* integer division */
"over" { return BOP_OVER; } /* binomial coefficient: n over k */
"in"  { return BOP_IN; }

{ID}	{
        // printf( "lex: An identifier: %s\n", xaxtext );
	CFunc *f1;
        if (!(f1= NFunctions::find(xaxtext))) {
	    printf("ERROR(lex) identifier %s is not a function\n", xaxtext);
	    return 0;
        }
	xaxlval->ptr = (void*) f1;
	return FNCT; }

"{"[^}\n]*"}"     /* eat up one-line comments */

[ \t\n]+          /* eat up whitespace */ 

"!"|"-"|"+"|"*"|"/"|"%"|"^"|"="|"("|")"|"["|"]"|","|"?"|":" {           
	// printf( "lex: Special character: %s\n", xaxtext );
	return xaxtext[0]; }

.       {           
	printf( "lex: invalid special character: %s\n", xaxtext);
	return 0; }

<<EOF>> { return END; }
%%

int xaxparse(void*);

int user_xaxparse(const char *lin, CTree *T)
{
	int err;

	xax_scan_string(lin);
	// printf( "/xax_scan\n" );

	err=xaxparse((void*) T);
	// printf( "/xax_parse\n" );

	xax_delete_buffer(YY_CURRENT_BUFFER);
	// printf( "/xax_delete\n" );

	return err;
}
