/* xax: eXtended Array eXpressions */

/* developed with GNU Bison 1.35 */
/* does not work with pre-GNU yacc */

%{
using namespace std;
#include <math.h>
#include <string.h>
#include <stdlib.h>
#include <string>
#include <vector>
#include "olm.h"

CRef *R;
%}


%union { /* stack data type YYSTYPE, used also by lexer */
double  val;
void*   ptr;
CTree*  tree;
CRef*   ref;
vector<CTree*>* vtree;
}

%pure-parser
%debug

%{
#define YYPARSE_PARAM T       /* used via xaxparse( CTree *T ) */
int xaxerror(const char*);    /* provided by calling code (expr.cpp, ...) */
int xaxlex(YYSTYPE *xaxlval); /* created by lex.l */
%}

%token <val>  NUM 
%token <ptr>  REF FRF FNCT END
%token <ptr>  BOP_EQ BOP_NE BOP_GT BOP_GE BOP_LT BOP_LE BOP_AND BOP_OR BOP_XOR BOP_OVER BOP_IN

%type  <tree>  exp rex
%type  <vtree> arr list
%type  <ref>   iva ifu /* {indexed} variable|function  */

/* Operator precedence */

%left '?' ':'
%left BOP_OR
%left BOP_AND
%left BOP_XOR
%left BOP_EQ BOP_NE
%left BOP_GT BOP_GE BOP_LT BOP_LE
%left '-' '+'
%left '*' '/' '%' BOP_MOD BOP_DIV
%left BOP_OVER /* binomial coefficient */
%left NEG     /* arithmetic negation (unary minus) */
%right '!'    /* logical not */
%right '^'    /* exponentiation */

/* Grammar follows */

%%
input: 
        /* empty */        { printf("pars got empty line\n"); }
      | line
;

line:
        exp END            { *((CTree*)T) = *$1;
	                     delete $1;
		             return 0; }
      | error              { yyerrok; return -1; }
;

exp:                       /* double-valued expression */
        NUM                { $$ = new CTree($1); }
      | iva                { $$ = new CTree($1); }
      | ifu '(' exp ')'    { $$ = new CTree($1, $3); }
/*    | VAR                { $$ = $1->value.var;              } */
/*    | VAR '=' exp        { $$ = $3; $1->value.var = $3;     } */
      | FNCT '(' exp ')'   { $$ = new CTree((CFunc*) $1, $3); }
      | FNCT '(' exp ',' exp ')' { $$ = new CTree((CFunc*) $1, $3, $5 ); }
      | FNCT '(' exp ',' exp ',' exp ')' { $$ = 
                                  new CTree((CFunc*) $1, $3, $5, $7 ); }
      | exp '+' exp        { $$ = new CTree(NFunctions::find("+"), $1, $3); }
      | exp '-' exp        { $$ = new CTree(NFunctions::find("-"), $1, $3); }
      | exp '*' exp        { $$ = new CTree(NFunctions::find("*"), $1, $3); }
      | exp '/' exp        { $$ = new CTree(NFunctions::find("/"), $1, $3); }
/*    | exp '%' exp        { $$ = new CTree(NFunctions::find("%"), $1, $3); }*/
      | exp '^' exp        { $$ = new CTree(NFunctions::find("^"), $1, $3); }
      | '-' exp  %prec NEG { $$ = new CTree(NFunctions::find("0-"), $2); }
      | exp BOP_EQ exp     { $$ = new CTree(NFunctions::find("=="),  $1, $3); }
      | exp BOP_NE exp     { $$ = new CTree(NFunctions::find("!="),  $1, $3); }
      | exp BOP_GT exp     { $$ = new CTree(NFunctions::find(">"),  $1, $3); }
      | exp BOP_GE exp     { $$ = new CTree(NFunctions::find(">="),  $1, $3); }
      | exp BOP_LT exp     { $$ = new CTree(NFunctions::find("<="),  $1, $3); }
      | exp BOP_LE exp     { $$ = new CTree(NFunctions::find("<"),  $1, $3); }
      | exp BOP_AND exp    { $$ = new CTree(NFunctions::find("&&"), $1, $3); }
      | exp BOP_OR exp     { $$ = new CTree(NFunctions::find("||"),  $1, $3); }
      | exp BOP_XOR exp    { $$ = new CTree(NFunctions::find("xor"), $1, $3); }
      | exp BOP_MOD exp    { $$ = new CTree(NFunctions::find("mod"), $1, $3); }
      | exp BOP_DIV exp    { $$ = new CTree(NFunctions::find("idiv"),$1, $3); }
      | exp BOP_OVER exp   { $$ = new CTree(NFunctions::find("over"),$1, $3); }
      | exp BOP_IN arr     { $$ = new CTree(NFunctions::find("in"),  $1, $3); }
      | '!' exp            { $$ = new CTree(NFunctions::find("!"), $2); }
      | exp '?' exp ':' exp { $$ = new CTree(NFunctions::find("?:"), 
                                             $1, $3, $5); }
      | '(' exp ')'        { $$ = $2; } 
;

arr:
      '[' list ']'         { $$ = $2; }
;

list:
        exp                { $$ = new vector<CTree*>; $$->push_back($1); }
      | list ',' exp       { $$ = $1; $$->push_back($3); }
;


iva:                       /* indexed variable */
        REF                { $$ = new CRef(string((const char*)$1)); }
      | REF '[' rex ']'    { $$ = new CRef(string((const char*)$1), $3); }
      | REF '[' rex ',' rex ']'   
                           { $$ = new CRef(string((const char*)$1), $3, $5); }
      | REF '[' rex ',' rex ',' rex ']'   
                          { $$ = new CRef(string((const char*)$1), $3,$5,$7); }
;

ifu:                       /* indexed function */
        FRF '[' rex ']'    { $$ = new CRef(string((const char*)$1), $3); }
      | FRF '[' rex ',' rex ']'   
                           { $$ = new CRef(string((const char*)$1), $3, $5); }
;

rex:                       /* reference expression */
        /* empty */        { $$ = 0;}
      | exp                { $$ = $1; }
;

/* End of grammar */
%%
